package com.tourMate.services.impl;


import com.tourMate.dao.TransportDao;
import com.tourMate.dto.*;
import com.tourMate.entities.*;
import com.tourMate.services.TransportManager;
import com.tourMate.services.UsersManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class TransportManagerImpl implements TransportManager {
    @Autowired
    TransportDao transportDao;

    @Autowired
    UsersManager usersManager;


    @Override
    public void createTransport(String transportName, String carBrand, String carType, int carManufacturedYear, int noPassengers, int noBags, long EMBG, Long userId, String carPlate) {
        User u = usersManager.findUserByID(userId);
        Transport t=new Transport(transportName,carBrand,carType,carManufacturedYear,noPassengers,noBags,EMBG,u,carPlate);
        transportDao.createTransport(t);
    }

    @Override
    public void deleteTransport(long transportId) {
        Transport t=getTransportById(transportId);
        transportDao.deleteTransport(t);
    }

    @Override
    public List<Transport> getTransports() {
        return transportDao.getTransports();
    }

    @Override
    public List<TransportDto> getTransportsByUser(long userId) {
        User u = usersManager.findUserByID(userId);
        List<Transport> transports = transportDao.getTransportsByUser(u);
        return transports.stream().map(x -> new TransportDto(
                x.getTransportID(),
                x.getTransportName(),
                x.getCarBrand(),
                x.getCarType(),
                x.getCarManufacturedYear(),
                x.getNoPassengers(),
                x.getNoBags(),
                x.getEMBG(),
                x.getOwner(),
                x.getCarPlate(),
                x.getAvailableRoutes().stream().map(y -> new RouteListingDto(
                        y.getTransportAvailibleId(),
                        y.getFrom(),
                        y.getTo(),
                        y.getDate(),
                        y.getFreeSpace(),
                        y.getTime(),
                        y.getRoutes(),
                        y.getRoutes().stream()
                                .mapToDouble(TransportRoute::getPrice)
                                .max().orElse(0)
                )).toList(),
                x.getAvailableRoutes().stream()
                        .flatMapToDouble(y -> y.getRoutes()
                                .stream()
                                .mapToDouble(TransportRoute::getPrice)).max().orElseGet(() -> 0)
        )).toList();
    }

    @Override
    public List<RouteListingDto> getRoutesForTransport(long transportId) {
        Transport t = getTransportById(transportId);
        List<TransportAvailible> transportAvailibles = transportDao.getRoutesForTransport(t);
        return transportAvailibles.stream().map(x -> new RouteListingDto(
                x.getTransportAvailibleId(),
                x.getFrom(),
                x.getTo(),
                x.getDate(),
                x.getFreeSpace(),
                x.getTime(),
                x.getRoutes(),
                x.getRoutes().stream()
                        .mapToDouble(TransportRoute::getPrice)
                        .max().orElse(0)
        )).toList();
    }

    @Override
    public TransportDto findTransportById(long transportId) {
        Transport x = transportDao.findTransportById(transportId);
        return new TransportDto(
                x.getTransportID(),
                x.getTransportName(),
                x.getCarBrand(),
                x.getCarType(),
                x.getCarManufacturedYear(),
                x.getNoPassengers(),
                x.getNoBags(),
                x.getEMBG(),
                x.getOwner(),
                x.getCarPlate(),
                x.getAvailableRoutes().stream().map(y -> new RouteListingDto(
                        y.getTransportAvailibleId(),
                        y.getFrom(),
                        y.getTo(),
                        y.getDate(),
                        y.getFreeSpace(),
                        y.getTime(),
                        y.getRoutes(),
                        y.getRoutes().stream()
                                .mapToDouble(TransportRoute::getPrice)
                                .max().orElse(0)
                )).toList(),
                x.getAvailableRoutes().stream()
                        .flatMapToDouble(y -> y.getRoutes()
                                .stream()
                                .mapToDouble(TransportRoute::getPrice)).max().orElseGet(() -> 0));
    }

    @Override
    public Transport getTransportById(Long transportId) {
        return transportDao.getTransportById(transportId);
    }

    @Override
    public void editTransport(long transportID, String transportName, String carBrand, String carType, int carManufacturedYear, int noPassengers, int noBags, long EMBG, User owner, String carPlate) {
        Transport t=getTransportById(transportID);
        t.setTransportName(transportName);
        t.setCarBrand(carBrand);
        t.setCarType(carType);
        t.setCarManufacturedYear(carManufacturedYear);
        t.setNoPassengers(noPassengers);
        t.setNoBags(noBags);
        t.setEMBG(EMBG);
        t.setOwner(owner);
        t.setCarPlate(carPlate);
        transportDao.editTransport(t);
    }

    @Override
    public void createTransportReservation(Long transportRouteId, Long userId, int noSeats) {
        TransportRoute transportRoute = transportDao.findTransportRouteById(transportRouteId);
        User u = usersManager.findUserByID(userId);
        TransportReservation transportReservation = new TransportReservation(transportRoute, noSeats, u);
        transportDao.createTransportReservation(transportReservation);
    }

    @Override
    public TransportReservation findTransportReservationByID(long reservationID) {
        return transportDao.findTransportReservationByID(reservationID);
    }

    @Override
    public List<TransportReservation> findTransportReservationByTransportId(long reservationID) {
        Transport t = getTransportById(reservationID);
        return transportDao.findTransportReservationByTransportId(t);
    }

    @Override
    public List<TransportReservation> getTransportReservations() {
        return transportDao.getTransportReservations();
    }

    @Override
    public TransportAvailible findTransportAvailableByID(long reservationID) {
        return transportDao.findTransportAvailableByID(reservationID);
    }

    @Override
    public Transport getTransportById(long transportId) {
        return transportDao.getTransportById(transportId);
    }

    @Override
    public List<Reviews> getReviewsForTransport(long transportId) {
        Transport transport = getTransportById(transportId);
        return transportDao.getReviewsForTransport(transport);
    }

    @Override
    public List<TransportListingDto> getTransportsAvailableByFilters(String from, String to, Date date, int numPassengers) {
        List<TransportRoute> transportAvailable = transportDao.getTransportsAvailableByFilters(from, to, date, numPassengers);
        Map<TransportAvailible, List<TransportRoute>> transportsByTransporter = transportAvailable.stream().collect(Collectors.groupingBy(x -> x.getParentRoute()));
        List<TransportListingDto> transportList = transportsByTransporter.keySet().stream().toList().stream()
                .map(x -> new TransportListingDto(
                        x.getTransportAvailibleId(),
                        x.getFrom(),
                        x.getTo(),
                        x.getDate(),
                        x.getFreeSpace(),
                        x.getTime(),
                        transportsByTransporter.get(x).stream().mapToDouble(y -> y.getPrice()).min().getAsDouble(),
                        x.getRoutes(),
                        x.getTransport(),
                        getReviewsForTransport(x.getTransport().getTransportID()),
                        getReviewsForTransport(x.getTransport().getTransportID()).stream().mapToDouble(Reviews::getNumStar).average().orElse(0)
                )).collect(Collectors.toList());
        return transportList;
    }

    @Override
    public List<TransportReservation> getTransportsReservationsByUserID(long userID) {
        return transportDao.getTransportsReservationsByUserID(userID);
    }

    @Override
    public List<TransportAvailible> getTransportsAvailable() {
        return transportDao.getTransportsAvailable();
    }

    @Override
    public void createTransportAvailable(TransportAvailible transportAvailible, Transport transport, String departureLocation, String arrivalLocation, Date date, Integer noSeats, Date departureHour, Long transportId, Collection<TransportRoute> trRoutes) {
        Transport t = getTransportById(transportId);
        List<TransportRoute> routes = trRoutes.stream().toList();
        routes.get(0).setDeparture(transportAvailible.getDate());
        transportAvailible.setTime(routes.get(routes.size() - 1).getArrival());
        routes.forEach(x -> x.setParentRoute(transportAvailible));
        transportAvailible.setTransport(t);
        TransportAvailible ta=new TransportAvailible(t,departureLocation,arrivalLocation,date,noSeats,departureHour);
        transportDao.createTransportAvailable(ta);
        routes.forEach(x -> {
            TransportRoute tr = new TransportRoute(ta, x.getFrom(), x.getTo(), x.getPrice(), x.getDeparture(), x.getArrival(), x.getFreeSpace(), x.getOrder());
            transportDao.createTransportRoute(tr);
        });
    }

    @Override
    public void editTransportAvailable(Transport transport, long availableID, String departureLocation, String arrivalLocation, Date date, Integer noSeats, Date departureHour) {
        TransportAvailible ta=findTransportAvailableByID(availableID);
        ta.setTransport(transport);
        ta.setFrom(departureLocation);
        ta.setTo(arrivalLocation);
        ta.setDate(date);
        ta.setFreeSpace(noSeats);
        ta.setTime(departureHour);
        transportDao.editTransportAvailable(ta);
    }

    @Override
    public void deleteTransportReservation(long reservationID) {
        TransportReservation tr=findTransportReservationByID(reservationID);
        transportDao.deleteTransportReservation(tr);
    }

    @Override
    public void deleteTransportAvailable(long availableID) {
        TransportAvailible ta=findTransportAvailableByID(availableID);
        transportDao.deleteTransportAvailable(ta);
    }

    @Override
    public void createTransportRoute(TransportAvailible parentRoute, String from, String to, double price, Date departure, Date arrival, int freeSpace, int order) {
        TransportRoute tr = new TransportRoute(parentRoute, from, to, price, departure, arrival, freeSpace, order);

        transportDao.createTransportRoute(tr);
    }

    @Override
    public void deleteTransportRoute(long transportRouteId) {
        TransportRoute tr = transportDao.findTransportRouteById(transportRouteId);
        transportDao.deleteTransportRoute(tr);
    }

    @Override
    public void editTransportRoute(long transportRouteId, TransportAvailible parentRoute, String from, String to, double price, Date departure, Date arrival, int freeSpace, int order) {
        TransportRoute tr = transportDao.findTransportRouteById(transportRouteId);
        tr.setParentRoute(parentRoute);
        tr.setFrom(from);
        tr.setTo(to);
        tr.setPrice(price);
        tr.setDeparture(departure);
        tr.setArrival(arrival);
        tr.setFreeSpace(freeSpace);
        tr.setOrder(order);
        transportDao.editTransportRoute(tr);
    }

    @Override
    public List<TransportReservationUserDto> findValidTransportReservationsByUser(Long userId) {
        User u = usersManager.findUserByID(userId);
        List<TransportReservation> reservations = transportDao.findReservationByUser(u);
        return reservations.stream().map(x -> new TransportReservationUserDto(
                x.getReservationID(),
                x.getUser(),
                x.getTransportRoute(),
                x.getTransportRoute().getParentRoute().getTransport().getTransportID(),
                x.getNoSeats(),
                x.getTransportRoute().getParentRoute().getTransport().getCarBrand(),
                x.getTransportRoute().getParentRoute().getTransport().getCarType(),
                x.getTransportRoute().getParentRoute().getTransport().getOwner(),
                x.getReviewed()
        )).toList();
    }

    @Override
    public List<TransportReservationUserDto> findPastTransportReservationsByUser(Long userId) {
        User u = usersManager.findUserByID(userId);
        List<TransportReservation> reservations = transportDao.findPastReservationByUser(u);
        return reservations.stream().map(x -> new TransportReservationUserDto(
                x.getReservationID(),
                x.getUser(),
                x.getTransportRoute(),
                x.getTransportRoute().getParentRoute().getTransport().getTransportID(),
                x.getNoSeats(),
                x.getTransportRoute().getParentRoute().getTransport().getCarBrand(),
                x.getTransportRoute().getParentRoute().getTransport().getCarType(),
                x.getTransportRoute().getParentRoute().getTransport().getOwner(),
                x.getReviewed()
        )).toList();
    }

    @Override
    public void setReservationReviewed(Long reservationId) {
        TransportReservation transportReservation = findTransportReservationByID(reservationId);
        transportReservation.setReviewed(true);
        transportDao.saveReservation(transportReservation);
    }

}


